package tfcustompane;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import oracle.adf.controller.internal.dtrt.binding.DTTaskFlowBinding;
import oracle.adf.model.BindingContext;
import oracle.adf.model.binding.DCBindingContainer;
import oracle.adf.model.binding.DCParameter;
import oracle.adf.share.ADFContext;
import oracle.adf.view.page.editor.context.ComposerContext;
import oracle.adf.view.page.editor.context.SelectionContext;
import oracle.adf.view.rich.component.rich.fragment.RichRegion;

import oracle.adfdt.ADFDesignTimeContext;
import oracle.adfdt.mds.MDSDesignTimeContext;
import oracle.adfdt.model.objects.CtrlParameter;
import oracle.adfdt.model.objects.Executable;
import oracle.adfdt.model.objects.PageDefinition;

import oracle.mds.core.MDSSession;


public class Util
{
  public static Map<String, String> getTaskFlowParameters(RichRegion region)
  {
    String tfId = _getTaskFlowBindingId(region);

    ComposerContext ctx = ComposerContext.getCurrentInstance();
    SelectionContext stx = ctx.getSelectionContext();
    String pagePath = stx.getSelectedComponent().getPagePath();


    Map<String, String> params = new HashMap<String, String>(4, 1);

    DCBindingContainer taskFlowBinding = _getTaskFlowBinding(pagePath, tfId);
    List<DCParameter> paramL = taskFlowBinding.getParametersList();

    if (paramL == null)
      return params;

    for (DCParameter param : paramL)
      params.put(param.getName(), param.getExpression());

    return params;
  }

  public static void setTaskFlowParameter(RichRegion region, String name,
                                          String value)
  {
    String tfId = _getTaskFlowBindingId(region);

    ComposerContext ctx = ComposerContext.getCurrentInstance();
    SelectionContext stx = ctx.getSelectionContext();
    String pagePath = stx.getSelectedComponent().getPagePath();

    DTTaskFlowBinding taskFlowBinding =
      (DTTaskFlowBinding) _getDTTaskFlowBinding(pagePath, tfId);

    CtrlParameter tfParam = taskFlowBinding.findParameter("parameter");

    if (tfParam == null)
      taskFlowBinding.addParameter(name, value);
    else
      tfParam.setExpression(value);
  }

  private static String _getTaskFlowBindingId(RichRegion region)
  {
    String value = region.getValueExpression(RichRegion.VALUE_KEY.getName()).
      getExpressionString();

    return value.substring(value.indexOf('.')+1, value.lastIndexOf('.'));
  }

  private static DCBindingContainer _getTaskFlowBinding(String pagePath,
                                                        String taskFlowId)
  {
    ADFContext adfCtx = ADFContext.getCurrent();
    BindingContext bindingCtx =
      (BindingContext) adfCtx.getSessionScope().get(BindingContext.CONTEXT_ID);

    DCBindingContainer dcBindCont =
      bindingCtx.findBindingContainerByPath(pagePath);
    return (DCBindingContainer) dcBindCont.findExecutableBinding(taskFlowId);
  }

  private static Executable _getDTTaskFlowBinding(String pagePath,
                                                  String taskFlowId)
  {
    ADFContext adfCtx = ADFContext.getCurrent();
    BindingContext bindingCtx =
      (BindingContext) adfCtx.getSessionScope().get(BindingContext.CONTEXT_ID);

    ADFDesignTimeContext _dtContext =
      new MDSDesignTimeContext((MDSSession) adfCtx.getMDSSessionAsObject(),
                               bindingCtx, pagePath);
    PageDefinition pageDef = _dtContext.findOrCreatePageDefinition();
    return pageDef.findExecutable(taskFlowId);
  }

}
